<?php
/**
 * XML_RPC5 - an xml-rpc implementation for PHP5
 * @package XML_RPC5
 * @author Greg Beaver <cellog@php.net>
 */
/**
 * This is not absolutely necessary, the exceptions could just as easily extend Exception
 */
require_once 'PEAR/Exception.php';
class Chiara_XML_RPC5_Exception extends PEAR_Exception {}
class Chiara_XML_RPC5_FaultException extends Chiara_XML_RPC5_Exception {}
class Chiara_XML_RPC5_CallException extends Chiara_XML_RPC5_Exception {}

/**
 * The base class for xml-rpc implementations.  use XML_RPC5::factory() to instantiate.
 *
 * This is backend code anyways, you probably will only need the Client/Server code.  This
 * class contains methods for encoding/decoding xml-rpc requests/responses.
 * <code>
 * <?php
 * $xmlrpc = XML_RPC5::factory();
 * ?>
 * </code>
 *
 * Child classes include XML_RPC5_php5 and XML_RPC_xmlrpc.  _php5 implements xml-rpc parsing and
 * generation without the use of the xmlrpc extension, whereas xmlrpc uses this extension.  Since
 * the extension is slightly faster, it will be loaded if present.
 * @package XML_RPC5
 * @author Greg Beaver <cellog@php.net>
 */
abstract class Chiara_XML_RPC5
{
    abstract public function encode($val);
    abstract public function encodeRequest($method, $val);
    abstract public function encodeResponse($val);
    abstract public function decode($val, $encoding = null);
    abstract public function decodeRequest($val, &$method, $encoding = null);
    abstract public function decodeResponse($val, $encoding = null);
    abstract public function getType($val);
    public function setType(&$val, $type)
    {
        if (!is_string($val)) {
            throw new Chiara_XML_RPC5_Exception('parameter $val must be a string to Chiara_XML_RPC5::setType()');
        }
    }
    /**
     * @return XML_RPC5
     */
    public static function factory()
    {
        if (extension_loaded('xmlrpc')) {
            include_once 'Chiara/XML/RPC5/xmlrpc.php';
            return new Chiara_XML_RPC5_xmlrpc;
        } else {
            include_once 'Chiara/XML/RPC5/php5.php';
            return new Chiara_XML_RPC5_php5;
        }
    }

    protected function validRequest($sxe, $method)
    {
        if (isset($sxe->fault)) {
            return true;
        } elseif ($method !== null && isset($sxe->methodName)) {
            if (isset($sxe->params)) {
                return $this->validParams($sxe->params);
            }
        } elseif ($method === null && isset($sxe->params)) {
            return $this->validParams($sxe->params);
        }
        return false;
    }

    protected function validParams($sxe)
    {
        $count = 0;
        foreach ($sxe->children() as $u) {
            $count++;
        }
        if (!$count) {
            return true;
        }
        if (!isset($sxe->param) || !isset($sxe->param->value)) {
            return false;
        }
        return true; // risky, but if the initial structure is OK, it's probably fine
    }
}
?>