<?php
class Chiara_XML_RPC5_xmlrpc extends Chiara_XML_RPC5
{
    public function encode($val)
    {
        return str_replace('&#10;', "\n", xmlrpc_encode($val));
    }

    public function encodeResponse($val)
    {
        return $this->encodeRequest(null, $val);
    }

    public function encodeRequest($method, $val)
    {
        return xmlrpc_encode_request($method, $val);
    }

    public function decode($val, $encoding = null)
    {
        $sxe = simplexml_load_string($val);
        if (!$sxe || !$this->validParams($sxe)) {
            throw new Chiara_XML_RPC5_Exception('Invalid xml input');
        }
        $ret = xmlrpc_decode($val, $encoding);
        if (xmlrpc_is_fault($ret)) {
            throw new Chiara_XML_RPC5_FaultException($ret['faultString'], $ret['faultCode']);
        }
        return $ret;
    }

    public function decodeRequest($val, &$method, $encoding = null)
    {
        $sxe = simplexml_load_string($val);
        if (!$sxe || !$this->validRequest($sxe, $method)) {
            throw new Chiara_XML_RPC5_Exception('Invalid xml input');
        }
        $ret = xmlrpc_decode_request($val, $method, $encoding);
        if (is_array($ret) && xmlrpc_is_fault($ret)) {
            throw new Chiara_XML_RPC5_FaultException($ret['faultString'], $ret['faultCode']);
        }
        return $ret;
    }

    public function decodeResponse($val, $encoding = null)
    {
        $null = null;
        return $this->decodeRequest($val, $null, $encoding);
    }

    public function getType($val)
    {
        return xmlrpc_get_type($val);
    }

    public function setType(&$val, $type)
    {
        parent::setType($val, $type);
        if ($type == 'datetime') {
            $value = strtotime($val);
            if ($value != -1) {
                $val = strftime('%Y%m%dT%H:%M:%S', $value);
            }
        }
        return xmlrpc_set_type($val, $type);
    }
}
?>